from dataclasses import dataclass
from enum import unique
from types import MappingProxyType

from poem_plugins.config.base import BaseConfig
from poem_plugins.general.strenum import StrEnum


@unique
class GitVersionFormatEnum(StrEnum):
    LONG = "long"
    SHORT = "short"


@dataclass
class GitProviderSettings(BaseConfig):
    MAPPERS = MappingProxyType(
        {
            "format": GitVersionFormatEnum,
            "version_prefix": str,
        },
    )

    format: GitVersionFormatEnum = GitVersionFormatEnum.SHORT
    version_prefix: str = "v"



from enum import Enum
from typing import Iterable


class StrEnum(str, Enum):
    def __str__(self) -> str:
        return self._value_

    @classmethod
    def choices(cls) -> Iterable[str]:
        return set(map(str, cls))



from typing import Any, Mapping

from cleo.events.console_events import COMMAND
from poetry.console.application import Application
from poetry.poetry import Poetry

from poem_plugins.config import VersionConfig
from poem_plugins.dispatchers.version import VersionDispatcher
from poem_plugins.plugins.base import BasePlugin


class VersionPlugin(BasePlugin):
    def get_raw_config(self, poetry: Poetry) -> Mapping[str, Any]:
        base_raw_config = super().get_raw_config(poetry)
        return base_raw_config.get("version", {})

    def get_config(self, poetry: Poetry) -> VersionConfig:
        base = self.get_raw_config(poetry)
        return VersionConfig.fabric(base)

    def activate(self, application: Application) -> None:
        if not application.event_dispatcher:
            return

        try:
            config = self.get_config(application.poetry)
        except Exception as exc:
            io = application.create_io()
            io.write_error_line(
                "<b>poem-plugins</b>: "
                f"Cannot load version config, skipping: {exc}",
            )
            return

        if not config.provider:
            return
        version_dispatcher = VersionDispatcher.factory(
            config=config,
        )
        application.event_dispatcher.add_listener(
            COMMAND,
            version_dispatcher,
        )



from dataclasses import dataclass, field
from enum import unique
from types import MappingProxyType
from typing import Optional

from poem_plugins.config.base import BaseConfig
from poem_plugins.config.git import GitProviderSettings
from poem_plugins.general.strenum import StrEnum


@unique
class VersionProviderEnum(StrEnum):
    GIT = "git"


@unique
class QuotesEnum(StrEnum):
    double = '"'
    single = "'"


@dataclass
class VersionConfig(BaseConfig):
    MAPPERS = MappingProxyType(
        {
            "provider": VersionProviderEnum,
            "update_pyproject": bool,
            "write_version_file": bool,
            "git": GitProviderSettings.fabric,
            "quote": QuotesEnum,
        },
    )

    provider: Optional[VersionProviderEnum] = None

    update_pyproject: bool = False
    write_version_file: bool = False
    version_file_quotes: Optional[QuotesEnum] = None

    git: GitProviderSettings = field(default_factory=GitProviderSettings)



from types import MappingProxyType
from typing import (
    Any, Callable, ClassVar, Mapping, MutableMapping, Type, TypeVar,
)


ConfigType = TypeVar("ConfigType", bound="BaseConfig")

KwargsType = Mapping[str, Any]

MapperType = Callable[[Any], Any]


def is_unset(value: Any) -> bool:
    return value in (None, "")


class BaseConfig:
    MAPPERS: ClassVar[Mapping[str, MapperType]] = MappingProxyType({})

    @classmethod
    def fabric(cls: Type[ConfigType], kwargs: KwargsType) -> ConfigType:
        mapped_kwargs: MutableMapping[str, Any] = {}
        for field, raw_value in kwargs.items():
            mapper = cls.MAPPERS.get(field)
            if not mapper:
                continue
            if is_unset(raw_value):
                mapped_value = None
            else:
                mapped_value = mapper(raw_value)
            mapped_kwargs[field] = mapped_value
        return cls(**mapped_kwargs)



from dataclasses import dataclass
from typing import Optional

from cleo.events.console_command_event import ConsoleCommandEvent
from cleo.events.event import Event
from cleo.events.event_dispatcher import EventDispatcher
from poetry.console.commands.build import BuildCommand
from poetry.core.utils.helpers import module_name
from poetry.poetry import Poetry
from tomlkit.toml_document import TOMLDocument

from poem_plugins.config import QuotesEnum, VersionConfig, VersionProviderEnum
from poem_plugins.general.version import Version
from poem_plugins.general.version.drivers import IVervsionDriver
from poem_plugins.general.version.drivers.git import GitVersionDriver


@dataclass(frozen=True)
class VersionDispatcher:
    config: VersionConfig
    driver: IVervsionDriver

    @classmethod
    def factory(cls, config: VersionConfig) -> "VersionDispatcher":
        if config.provider == VersionProviderEnum.GIT:
            driver = GitVersionDriver(settings=config.git)
        else:
            driver = GitVersionDriver(settings=config.git)
        return cls(
            driver=driver,
            config=config,
        )

    def _write_pyproject(
        self,
        poetry: Poetry,
        version: Version,
    ) -> None:
        if not self.config.update_pyproject:
            return
        content: TOMLDocument = poetry.file.read()
        poetry_content = content["tool"]["poetry"]  # type: ignore
        poetry_content["version"] = str(version)  # type: ignore
        poetry.file.write(content)

    def _write_module(
        self,
        poetry: Poetry,
        version: Version,
        quotes: Optional[QuotesEnum] = None,
    ) -> None:
        if not self.config.write_version_file:
            return
        package_name = module_name(poetry.package.name)
        with open(f"{package_name}/version.py", "w") as file:
            content = self.driver.render_version_file(version=version)
            if quotes is not None:
                content = content.replace('"', quotes)
            file.write(content)

    def __call__(
        self,
        event: Event,
        event_name: str,
        dispatcher: EventDispatcher,
    ) -> None:
        if not isinstance(event, ConsoleCommandEvent):
            return
        command = event.command
        if not isinstance(command, BuildCommand):
            return
        io = event.io
        poetry = command.poetry
        try:
            version = self.driver.get_version()
        except Exception as exc:
            io.write_error_line(f"<b>poem-plugins</b>: {exc}")
            raise exc

        io.write_line(
            f"<b>poem-plugins</b>: Setting version to: {version}",
        )
        poetry.package.version = str(version)  # type: ignore

        self._write_pyproject(poetry, version)
        self._write_module(poetry, version, self.config.version_file_quotes)



import re
import subprocess
import warnings
from dataclasses import dataclass
from shutil import which
from types import MappingProxyType
from typing import Any, Callable, ClassVar, Mapping, Match, Optional

from poem_plugins.config.git import GitProviderSettings, GitVersionFormatEnum
from poem_plugins.general.version import Version
from poem_plugins.general.version.drivers import IVervsionDriver


GIT_BIN = which("git")
WARNING_TEXT = (
    '"git" binary was not found, this plugin this will not work properly'
)
if GIT_BIN is None:
    warnings.warn(WARNING_TEXT)


@dataclass
class GitVersionDriver(IVervsionDriver):
    settings: GitProviderSettings

    VERSION_TEMPLATE: str = (
        "# THIS FILE WAS GENERATED AUTOMATICALLY\n"
        "# BY: {whoami}\n"
        "# NEVER EDIT THIS FILE MANUALLY\n"
        "\n"
        "version_info = ({major}, {minor}, {patch})\n"
        '__version__ = "{version}"\n'
    )

    CONVERTERS: ClassVar[
        Mapping[str, Callable[[Any], Any]]
    ] = MappingProxyType(
        {
            "major": int,
            "minor": int,
            "patch": int,
        },
    )

    def get_version(self) -> Version:
        if GIT_BIN is None:
            raise RuntimeError(WARNING_TEXT)

        result = subprocess.run(
            [GIT_BIN, "describe", "--long"],
            stdout=subprocess.PIPE,
            text=True,
        )

        if result.returncode != 0:
            raise RuntimeError("Cannot found git version")
        raw_version = result.stdout.strip()
        regex = (
            r"(?P<major>\d+)\.(?P<minor>\d+)-(?P<patch>\d+)-(?P<commit>\S+)"
        )
        match: Optional[Match[str]] = re.match(
            self.settings.version_prefix + regex,
            raw_version,
        )

        if match is None:
            raise RuntimeError("Cannot parse git version")
        raw_kwargs = dict(match.groupdict())
        if self.settings.format == GitVersionFormatEnum.SHORT:
            raw_kwargs.pop("commit", None)
        kwargs = {
            k: self.CONVERTERS.get(k, lambda x: x)(v)
            for k, v in raw_kwargs.items()
        }

        return Version(**kwargs)

    def render_version_file(
        self,
        version: Version,
    ) -> str:
        return self.VERSION_TEMPLATE.format(
            whoami='poem-plugins "git" plugin',
            major=version.major,
            minor=version.minor,
            patch=version.patch,
            version=str(version),
        )



import abc
from typing import Any, Mapping

from poetry.plugins.application_plugin import ApplicationPlugin
from poetry.poetry import Poetry

from poem_plugins.config import BaseConfig


class BasePlugin(ApplicationPlugin, abc.ABC):
    @abc.abstractmethod
    def get_config(self, poetry: Poetry) -> BaseConfig:
        raise NotImplementedError

    def get_raw_config(self, poetry: Poetry) -> Mapping[str, Any]:
        return poetry.pyproject.data.get("tool", {}).get("poem-plugins", {})



import abc

from poem_plugins.general.version import Version


class IVervsionDriver(abc.ABC):
    @abc.abstractmethod
    def get_version(self) -> Version:
        raise NotImplementedError

    @abc.abstractmethod
    def render_version_file(
        self, version: Version,
    ) -> str:
        raise NotImplementedError
